import datetime

from sqlalchemy import DateTime, TypeDecorator

UTC = datetime.timezone.utc


class UTCDateTime(TypeDecorator):
    """
    Custom type decorator to convert naive datetime to UTC timezone-aware datetime on
    load.
    """

    impl = DateTime

    def process_bind_param(self, value, dialect):
        # If the value is already aware, make sure it is converted to UTC
        if value is not None and value.utcoffset() is not None:
            return value.astimezone(UTC)
        return value

    def process_result_value(self, value, dialect):
        # Convert naive datetime to UTC timezone-aware datetime
        if value is not None and value.utcoffset() is None:
            return value.replace(tzinfo=UTC)
        return value
